function [FOCA,mFOCA,mFOCA2,Ct,Cs] = Dong_FOCA(data,mask,TR,ts,flag)
% Calsulating spatial temporal index: the FOur dimensional (spatio-temporal)
% Consistency of local neural Activities (FOCA) around each voxel.
% Input:
%      data: Cleaned 4D data; X x Y x Z x timgN e.g. 53 X 63 X 46 X timeN. 
%            The nuisance singals such as headmotion, linear drift, white and 
%            CSF signals should be removed from data before caulating FOCA.
%      mask: brain mask: X x Y x Z: e.g. 53 X 63 X 46
%      TR: Repeating Time of fMRI
%      ts: neiborghing times.Default is equal to TR time.
%      flagflag = 0 -> local 27 voxels (default); flag = 1 -> local 19 voxels; 
%            flag = 2 -> local 7 voxels.
% Output:
%      FOCA: spatial-temporal index
%      mFOCA: normalized spatial-temporal index (Divided by the average FOCA)
%      mFOCA2: mFOCA - 1;
%      Ct: temporal correlation in local region(e.g. 27 voxels)
%      Cs: spatial correlation in neighboring time (-ts - +ts)
% -------------------------------------------------------------------------
% The package is written by Li Dong: Li_dong729@163.com
% last edit 2014.11.24
% -------------------------------------------------------------------------
if nargin < 3
    error('stats:FOCA:TooFewInputs','Requires four arguments.');
elseif nargin == 3
    ts = TR;
    flag = 0;
elseif nargin == 4
    flag = 0;
end
% ---------------------
sizedata = size(data); % size of data
timeN = sizedata(4); % length of time point
DIM = sizedata(1:3); % data dimension
mask1 = mask(:); % brainmask
brainIndex=find(mask1>0);
DATA = zeros(size(brainIndex,1),timeN);
for j = 1:timeN
    data1 = data(:,:,:,j);
    DATA(:,j) = data1(mask==1);
end;

Ct = zeros(DIM);% temporal corrlation
Cs = zeros(DIM); % spatial correlation
FOCA = zeros(DIM); % spatial-temporal index
mFOCA = zeros(DIM); % normalized spatial-temporal index
mFOCA2 = zeros(DIM); % mFOCA - 1.
Ts = floor(ts/TR); % neiborghing time pointsTR
% --------------------------
% calculating
for i = 1:size(brainIndex,1)
    [x,y,z] = ind2sub(DIM,brainIndex(i));
    % -----------------------------------------
    [cluster_Index,ind2] = find_cluster(DIM,brainIndex(i),mask,flag);
    ind1 = ismember(brainIndex,cluster_Index);
    data2 = DATA(ind1,:);data2=data2';
    data3 = data2(:,any(data2,1)); % removing voxels whose values are 0.
    % ---------------------------------------------
    % temporal correlation
    if size(data3,2) <= 2
        Ct(x,y,z) = 0.0001;
    else
        rr1 = corrcoef(data3);
        rr2 = rr1(triu(ones(size(rr1,1)),1)==1);
        rr2(~isfinite(rr2)) = [];   % Toss NaN's
        r1 = abs(mean(rr2));
        Ct(x,y,z) = r1;
    end
    % ---------------------------------------------
    % spatial correlation
    if size(data3,2) <= 5
        Cs(x,y,z) = 0.0001;
    else
        rr3 = corrcoef(data3');
        ix = triu(ones(timeN),-Ts) - triu(ones(timeN),Ts+1) - eye(timeN);
        ix(~isfinite(rr3)) = 0;% Toss NaN's
        rr4 = rr3.*ix;
        r2 = abs(mean(sum(rr4,1)./sum(ix,1)));
        Cs(x,y,z) = r2;
        % -------------------------------------
    end
end;
Ct(~isfinite(Ct)) = 0; % Toss NaN's
Cs(~isfinite(Cs)) = 0; % Toss NaN's
% ---------------------------------------------
FOCA = Ct .* Cs; % multiply
mFOCA = FOCA./(sum(FOCA(:)/sum(mask1)));
mFOCA2 = mFOCA - mask;
% -------------------------------------------------------------------------
end

% =========================================================================
% sub function
function [cluster_Index,core_index] = find_cluster(DIM,Nv,mask,flag)
% finding local voxels.
% Input
%      DIM data dimentiome.g. 53 X 63 X 46
%      Nv: i-th voxel
%      mask brain mask
%      flagflag = 0 --> local 27 voxelsflag = 1 --> local 19 voxelsflag
%      = 2 --> local 7 voxels
% Output:
%      cluster_Index: the locations of neighboring voxels in one demension
%                     (containing itself).
%      core_indexlocation of center voxel in cluster_index.
cluster_Index = [];
[X,Y,Z]=ind2sub(DIM,Nv);

xi = max(X-1,1):min(X+1,DIM(1));
yi = max(Y-1,1):min(Y+1,DIM(2));
zi = max(Z-1,1):min(Z+1,DIM(3));

xyz = [];
xyz(1,:) = repmat(xi,1,length(yi)*length(zi));

y = repmat(yi,length(xi),1); y=(y(:))';
xyz(2,:) = repmat(y,1,length(zi));

z = repmat(zi,length(yi)*length(xi),1);
xyz(3,:) = (z(:))';
switch flag
    
    case 0  % 27 voxels
        % ----------------
        mask1 = mask(xi,yi,zi);
        block_mask = mask1(:);
        index1 = sub2ind(DIM,xyz(1,:),xyz(2,:),xyz(3,:));
        cluster_Index = index1(block_mask == 1);
        core_index = find(cluster_Index==Nv);
        
    case 1 % 19 voxels
        % ------------
        xx1 = max(X-1,1);xx2 = min(X+1,DIM(1));
        yy1 = max(Y-1,1);yy2 = min(Y+1,DIM(2));
        zz1 = max(Z-1,1);zz2 = min(Z+1,DIM(3));
        % -------------------
        mask2 = mask;
        mask2(xx1,yy1,zz1)=0;mask2(xx1,yy2,zz1)=0;
        mask2(xx2,yy1,zz1)=0;mask2(xx2,yy2,zz1)=0;
        mask2(xx1,yy1,zz2)=0;mask2(xx1,yy2,zz2)=0;
        mask2(xx2,yy1,zz2)=0;mask2(xx2,yy2,zz2)=0;
        % -----------------------
        mask1 = mask2(xi,yi,zi);
        block_mask = mask1(:);
        index1 = sub2ind(DIM,xyz(1,:),xyz(2,:),xyz(3,:));
        cluster_Index = index1(block_mask == 1);
        core_index = find(cluster_Index==Nv);
    case 2 % 7 voxels
        xx1 = max(X-1,1);xx2 = min(X+1,DIM(1));
        yy1 = max(Y-1,1);yy2 = min(Y+1,DIM(2));
        zz1 = max(Z-1,1);zz2 = min(Z+1,DIM(3));
        % -----------------
        mask2 = mask;
        mask2(xx1,yy1,zz1)=0;mask2(xx1,yy2,zz1)=0;
        mask2(xx2,yy1,zz1)=0;mask2(xx2,yy2,zz1)=0;
        mask2(xx1,yy1,zz2)=0;mask2(xx1,yy2,zz2)=0;
        mask2(xx2,yy1,zz2)=0;mask2(xx2,yy2,zz2)=0;
        
        mask2(X,yy1,zz1)=0;mask2(xx1,Y,zz1)=0;
        mask2(X,yy2,zz1)=0;mask2(xx2,Y,zz1)=0;
        mask2(X,yy1,zz2)=0;mask2(xx1,Y,zz2)=0;
        mask2(X,yy2,zz2)=0;mask2(xx2,Y,zz2)=0;
        mask2(xx1,yy1,Z)=0;mask2(xx1,yy2,Z)=0;
        mask2(xx2,yy1,Z)=0;mask2(xx2,yy2,Z)=0;
        % -----------------
        mask1 = mask2(xi,yi,zi);
        block_mask = mask1(:);
        index1 = sub2ind(DIM,xyz(1,:),xyz(2,:),xyz(3,:));
        cluster_Index = index1(block_mask == 1);
        core_index = find(cluster_Index==Nv);
end
end



